﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;

namespace iTrackWSTest
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length < 2)
            {
                Console.WriteLine("Usage: iTrackWSTest.exe username password");
                Console.ReadKey();
                return;
            }
            String userName = args[0];
            String password = args[1];

            iTrackServiceReference.DefaultWebServicePortTypeClient iTrackWS = new iTrackWSTest.iTrackServiceReference.DefaultWebServicePortTypeClient("DefaultWebServiceHttpSoap11Endpoint");
            String sessionId = "";

            Console.WriteLine("Try to get any data without logging in or with an old sessionid:");
            try
            {
                iTrackWS.getTargets(sessionId);
            }
            catch (Exception e)
            {
                Console.WriteLine("Error on getting data!");
            }

            //logging in
            try
            {
                Console.WriteLine("Logging in with wrong password:");
                sessionId = iTrackWS.login(userName, "wrongpassword");
                Console.WriteLine("Login successful");
            }
            catch (Exception e)
            {
                Console.WriteLine("Login failed!");
            }

            try
            {
                Console.WriteLine("Logging in with right password:");
                sessionId = iTrackWS.login(userName, password);
                Console.WriteLine("Login successful");

                //get the target list
                Console.WriteLine("Getting target list:");
                String targetsXml = iTrackWS.getTargets(sessionId);

                //parsing xml
                XmlDocument xmldoc = new XmlDocument();
                xmldoc.LoadXml(targetsXml);
                XmlElement root = xmldoc.DocumentElement;
                XmlNodeList childlist = root.ChildNodes;
                int?[] targetIds = new int?[childlist.Count];
                for (int i = 0; i < childlist.Count; i++)
                {
                    XmlNode target = childlist.Item(i);
                    String name = target.Attributes.GetNamedItem("name").Value;
                    String description = target.Attributes.GetNamedItem("description").Value;
                    targetIds[i] = Convert.ToInt32(target.Attributes.GetNamedItem("id").Value);
                    Console.WriteLine("Target id=" + targetIds[i] + " name=" + name + " description=" + description);
                }


                //getting last positions
                Console.WriteLine("Getting last positions:");
                /* if the answer contains big amount of data, please increase the limit of maxStringContentLength in app.config */
                String positionsXml = iTrackWS.getLastCoordinates(sessionId, targetIds);


                //parsing xml
                xmldoc.LoadXml(positionsXml);
                root = xmldoc.DocumentElement;
                childlist = root.ChildNodes;
                TimeZoneInfo tzCEST = TimeZoneInfo.FindSystemTimeZoneById("Central European Standard Time");
                long epochticks = (new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc)).Ticks;
                for (int i = 0; i < childlist.Count; i++)
                {
                    XmlNode target = childlist.Item(i);
                    int id = Convert.ToInt32(target.Attributes.GetNamedItem("target-id").Value);
                    //time value in microseconds since epoch (January 1, 1970, 00:00:00 CEST)
                    long timeval = Convert.ToInt64(target.Attributes.GetNamedItem("time").Value);
                    DateTime time = TimeZoneInfo.ConvertTimeFromUtc(new DateTime(timeval * 10 + epochticks, DateTimeKind.Utc), tzCEST);
                    string longitude = target.Attributes.GetNamedItem("longitude").Value;
                    string latitude = target.Attributes.GetNamedItem("latitude").Value;
                    bool engineon = Convert.ToBoolean(target.Attributes.GetNamedItem("engineon").Value);
                    int speed = Convert.ToInt32(target.Attributes.GetNamedItem("speed").Value);

                    Console.WriteLine("Last status of target(id=" + id + "):" +
                            " time=\"" + time + "\"" +
                            " longitude=" + longitude + " latitude=" + latitude +
                            " engineon=" + engineon + " speed=" + speed);
                }

                //logging out
                iTrackWS.logout(sessionId);
                Console.WriteLine("Logged out");
                Console.ReadKey();
            }
            catch (Exception e)
            {
                Console.WriteLine("Some error occured. Please check your internet connection! " + e.Message);
                Console.ReadKey();
            }
        }
    }
}
